From: Joerg Roedel <jroe...@suse.de>

Factor out the code to fetch the instruction from user-space to a helper
function.

No functional changes.

Signed-off-by: Joerg Roedel <jroe...@suse.de>
Link: https://lore.kernel.org/r/20200724160336.5435-8-j...@8bytes.org
---
 arch/x86/include/asm/insn-eval.h |  2 ++
 arch/x86/kernel/umip.c           | 26 +++++-----------------
 arch/x86/lib/insn-eval.c         | 38 ++++++++++++++++++++++++++++++++
 3 files changed, 46 insertions(+), 20 deletions(-)

diff --git a/arch/x86/include/asm/insn-eval.h b/arch/x86/include/asm/insn-eval.h
index 2b6ccf2c49f1..b8b9ef1bbd06 100644
--- a/arch/x86/include/asm/insn-eval.h
+++ b/arch/x86/include/asm/insn-eval.h
@@ -19,5 +19,7 @@ void __user *insn_get_addr_ref(struct insn *insn, struct 
pt_regs *regs);
 int insn_get_modrm_rm_off(struct insn *insn, struct pt_regs *regs);
 unsigned long insn_get_seg_base(struct pt_regs *regs, int seg_reg_idx);
 int insn_get_code_seg_params(struct pt_regs *regs);
+int insn_fetch_from_user(struct pt_regs *regs,
+                        unsigned char buf[MAX_INSN_SIZE]);
 
 #endif /* _ASM_X86_INSN_EVAL_H */
diff --git a/arch/x86/lib/insn-eval.c b/arch/x86/lib/insn-eval.c
index 31600d851fd8..0c4f7ebc261b 100644
--- a/arch/x86/lib/insn-eval.c
+++ b/arch/x86/lib/insn-eval.c
@@ -1369,3 +1369,41 @@ void __user *insn_get_addr_ref(struct insn *insn, struct 
pt_regs *regs)
                return (void __user *)-1L;
        }
 }
+
+/**
+ * insn_fetch_from_user() - Copy instruction bytes from user-space memory
+ * @regs:      Structure with register values as seen when entering kernel mode
+ * @buf:       Array to store the fetched instruction
+ *
+ * Gets the linear address of the instruction and copies the instruction bytes
+ * to the buf.
+ *
+ * Returns:
+ *
+ * Number of instruction bytes copied.
+ *
+ * 0 if nothing was copied.
+ */
+int insn_fetch_from_user(struct pt_regs *regs, unsigned char 
buf[MAX_INSN_SIZE])
+{
+       unsigned long seg_base = 0;
+       int not_copied;
+
+       /*
+        * If not in user-space long mode, a custom code segment could be in
+        * use. This is true in protected mode (if the process defined a local
+        * descriptor table), or virtual-8086 mode. In most of the cases
+        * seg_base will be zero as in USER_CS.
+        */
+       if (!user_64bit_mode(regs)) {
+               seg_base = insn_get_seg_base(regs, INAT_SEG_REG_CS);
+               if (seg_base == -1L)
+                       return 0;
+       }
+
+
+       not_copied = copy_from_user(buf, (void __user *)(seg_base + regs->ip),
+                                   MAX_INSN_SIZE);
+
+       return MAX_INSN_SIZE - not_copied;
+}
diff --git a/arch/x86/kernel/umip.c b/arch/x86/kernel/umip.c
index 2c304fd0bb1a..ad135be4f1f0 100644
--- a/arch/x86/kernel/umip.c
+++ b/arch/x86/kernel/umip.c
@@ -335,11 +335,11 @@ static void force_sig_info_umip_fault(void __user *addr, 
struct pt_regs *regs)
  */
 bool fixup_umip_exception(struct pt_regs *regs)
 {
-       int not_copied, nr_copied, reg_offset, dummy_data_size, umip_inst;
-       unsigned long seg_base = 0, *reg_addr;
+       int nr_copied, reg_offset, dummy_data_size, umip_inst;
        /* 10 bytes is the maximum size of the result of UMIP instructions */
        unsigned char dummy_data[10] = { 0 };
        unsigned char buf[MAX_INSN_SIZE];
+       unsigned long *reg_addr;
        void __user *uaddr;
        struct insn insn;
        int seg_defs;
@@ -347,26 +347,12 @@ bool fixup_umip_exception(struct pt_regs *regs)
        if (!regs)
                return false;
 
-       /*
-        * If not in user-space long mode, a custom code segment could be in
-        * use. This is true in protected mode (if the process defined a local
-        * descriptor table), or virtual-8086 mode. In most of the cases
-        * seg_base will be zero as in USER_CS.
-        */
-       if (!user_64bit_mode(regs))
-               seg_base = insn_get_seg_base(regs, INAT_SEG_REG_CS);
-
-       if (seg_base == -1L)
-               return false;
-
-       not_copied = copy_from_user(buf, (void __user *)(seg_base + regs->ip),
-                                   sizeof(buf));
-       nr_copied = sizeof(buf) - not_copied;
+       nr_copied = insn_fetch_from_user(regs, buf);
 
        /*
         * The copy_from_user above could have failed if user code is protected
         * by a memory protection key. Give up on emulation in such a case.
         * Should we issue a page fault?
         */
        if (!nr_copied)
                return false;
@@ -347,26 +347,12 @@ bool fixup_umip_exception(struct pt_regs *regs)
 
        /*
-        * The copy_from_user above could have failed if user code is protected
-        * by a memory protection key. Give up on emulation in such a case.
-        * Should we issue a page fault?
+        * The insn_fetch_from_user above could have failed if user code
+        * is protected by a memory protection key. Give up on emulation
+        * in such a case.  Should we issue a page fault?
         */
        if (!nr_copied)
-- 
2.28.0

_______________________________________________
Virtualization mailing list
Virtualization@lists.linux-foundation.org
https://lists.linuxfoundation.org/mailman/listinfo/virtualization

Reply via email to